// AI Final Project
// Module: JFugue Daemon
// CSC 466 - Graci
// Jacob Peck - 20110426

import java.nio.file.*;
import static java.nio.file.StandardWatchEventKind.*;
import static java.nio.file.LinkOption.*;
import java.nio.file.attribute.*;
import java.io.*;
import java.util.*;
import org.jfugue.*;

public class JFugueDaemon {
  private static WatchService watcher;
  private static WatchKey watchkey;
  private static Player player;
  private static Scanner sc;
  
  static <T> WatchEvent<T> castEvent(WatchEvent<?> event) {
    return (WatchEvent<T>)event;
  }
  
  public static void main(String[] args) throws Exception {
    // register directory and process its events
    Path dir = Paths.get("temp");
    watcher = FileSystems.getDefault().newWatchService();
    watchkey = dir.register(watcher, ENTRY_CREATE);
    
    // other vars
    player = new Player();
    String separator = System.getProperty("file.separator");
    
    System.out.println("JFugue Daemon v. 20110427, by Jacob Peck");
    System.out.println("Watches the ./temp/ directory for created files,");
    System.out.println("plays them, and saves them as a .midi file in ./output/");
    System.out.println("Press ^C to close.");
    
    while (true) {
      watchkey = watcher.take();
      for (WatchEvent<?> event: watchkey.pollEvents()) {
        WatchEvent<Path> watchEvent = castEvent(event);
        // events here
        String filename = dir.resolve(watchEvent.context()).toString();
        System.out.println("Filename: " + filename);
        
        // get contents
        boolean success = false;
        String contents = "";
        do {
          sc = new Scanner(new File(filename));
          sc.reset(); // prevent crash (weird Java 1.7.0-ea bug?)
          if (sc.hasNextLine()) {
            contents = sc.nextLine();
            success = true;
          }
        } while (!success);
        sc.close();
        System.out.println("Contents: " + contents);
        
        // JFugue magic
        player.play(contents);
        filename = "output" + separator + 
                   filename.split(java.util.regex.Pattern.quote(separator))[1].trim() + ".midi";
        player.saveMidi(contents, new File(filename));
        System.out.println("File saved as: " + filename);
        
        // reset watcher
        watchkey.reset();
      }
    }
    
  }
  
}